@@ -16,29 +16,33 @@ classification.
 # - Replace link module of GLMs.
 
 import numbers
+
 import numpy as np
 from scipy.special import xlogy
+
+from ..utils import check_scalar
+from ..utils.stats import _weighted_percentile
 from ._loss import (
-    CyHalfSquaredError,
     CyAbsoluteError,
-    CyPinballLoss,
-    CyHalfPoissonLoss,
+    CyExponentialLoss,
+    CyHalfBinomialLoss,
     CyHalfGammaLoss,
+    CyHalfMultinomialLoss,
+    CyHalfPoissonLoss,
+    CyHalfSquaredError,
     CyHalfTweedieLoss,
     CyHalfTweedieLossIdentity,
-    CyHalfBinomialLoss,
-    CyHalfMultinomialLoss,
+    CyHuberLoss,
+    CyPinballLoss,
 )
 from .link import (
-    Interval,
+    HalfLogitLink,
     IdentityLink,
-    LogLink,
+    Interval,
     LogitLink,
+    LogLink,
     MultinomialLogit,
 )
-from ..utils import check_scalar
-from ..utils._readonly_array_wrapper import ReadonlyArrayWrapper
-from ..utils.stats import _weighted_percentile
 
 
 # Note: The shape of raw_prediction for multiclass classifications are
@@ -185,10 +189,6 @@ class BaseLoss:
         if raw_prediction.ndim == 2 and raw_prediction.shape[1] == 1:
             raw_prediction = raw_prediction.squeeze(1)
 
-        y_true = ReadonlyArrayWrapper(y_true)
-        raw_prediction = ReadonlyArrayWrapper(raw_prediction)
-        if sample_weight is not None:
-            sample_weight = ReadonlyArrayWrapper(sample_weight)
         return self.closs.loss(
             y_true=y_true,
             raw_prediction=raw_prediction,
@@ -250,10 +250,6 @@ class BaseLoss:
         if gradient_out.ndim == 2 and gradient_out.shape[1] == 1:
             gradient_out = gradient_out.squeeze(1)
 
-        y_true = ReadonlyArrayWrapper(y_true)
-        raw_prediction = ReadonlyArrayWrapper(raw_prediction)
-        if sample_weight is not None:
-            sample_weight = ReadonlyArrayWrapper(sample_weight)
         return self.closs.loss_gradient(
             y_true=y_true,
             raw_prediction=raw_prediction,
@@ -303,10 +299,6 @@ class BaseLoss:
         if gradient_out.ndim == 2 and gradient_out.shape[1] == 1:
             gradient_out = gradient_out.squeeze(1)
 
-        y_true = ReadonlyArrayWrapper(y_true)
-        raw_prediction = ReadonlyArrayWrapper(raw_prediction)
-        if sample_weight is not None:
-            sample_weight = ReadonlyArrayWrapper(sample_weight)
         return self.closs.gradient(
             y_true=y_true,
             raw_prediction=raw_prediction,
@@ -371,10 +363,6 @@ class BaseLoss:
         if hessian_out.ndim == 2 and hessian_out.shape[1] == 1:
             hessian_out = hessian_out.squeeze(1)
 
-        y_true = ReadonlyArrayWrapper(y_true)
-        raw_prediction = ReadonlyArrayWrapper(raw_prediction)
-        if sample_weight is not None:
-            sample_weight = ReadonlyArrayWrapper(sample_weight)
         return self.closs.gradient_hessian(
             y_true=y_true,
             raw_prediction=raw_prediction,
@@ -600,7 +588,7 @@ class PinballLoss(BaseLoss):
     Additional Attributes
     ---------------------
     quantile : float
-        The quantile to be estimated. Must be in range (0, 1).
+        The quantile level of the quantile to be estimated. Must be in range (0, 1).
     """
 
     differentiable = False
@@ -636,6 +624,79 @@ class PinballLoss(BaseLoss):
             )
 
 
+class HuberLoss(BaseLoss):
+    """Huber loss, for regression.
+
+    Domain:
+    y_true and y_pred all real numbers
+    quantile in (0, 1)
+
+    Link:
+    y_pred = raw_prediction
+
+    For a given sample x_i, the Huber loss is defined as::
+
+        loss(x_i) = 1/2 * abserr**2            if abserr <= delta
+                    delta * (abserr - delta/2) if abserr > delta
+
+        abserr = |y_true_i - raw_prediction_i|
+        delta = quantile(abserr, self.quantile)
+
+    Note: HuberLoss(quantile=1) equals HalfSquaredError and HuberLoss(quantile=0)
+    equals delta * (AbsoluteError() - delta/2).
+
+    Additional Attributes
+    ---------------------
+    quantile : float
+        The quantile level which defines the breaking point `delta` to distinguish
+        between absolute error and squared error. Must be in range (0, 1).
+
+     Reference
+    ---------
+    .. [1] Friedman, J.H. (2001). :doi:`Greedy function approximation: A gradient
+      boosting machine <10.1214/aos/1013203451>`.
+      Annals of Statistics, 29, 1189-1232.
+    """
+
+    differentiable = False
+    need_update_leaves_values = True
+
+    def __init__(self, sample_weight=None, quantile=0.9, delta=0.5):
+        check_scalar(
+            quantile,
+            "quantile",
+            target_type=numbers.Real,
+            min_val=0,
+            max_val=1,
+            include_boundaries="neither",
+        )
+        self.quantile = quantile  # This is better stored outside of Cython.
+        super().__init__(
+            closs=CyHuberLoss(delta=float(delta)),
+            link=IdentityLink(),
+        )
+        self.approx_hessian = True
+        self.constant_hessian = False
+
+    def fit_intercept_only(self, y_true, sample_weight=None):
+        """Compute raw_prediction of an intercept-only model.
+
+        This is the weighted median of the target, i.e. over the samples
+        axis=0.
+        """
+        # See formula before algo 4 in Friedman (2001), but we apply it to y_true,
+        # not to the residual y_true - raw_prediction. An estimator like
+        # HistGradientBoostingRegressor might then call it on the residual, e.g.
+        # fit_intercept_only(y_true - raw_prediction).
+        if sample_weight is None:
+            median = np.percentile(y_true, 50, axis=0)
+        else:
+            median = _weighted_percentile(y_true, sample_weight, 50)
+        diff = y_true - median
+        term = np.sign(diff) * np.minimum(self.closs.delta, np.abs(diff))
+        return median + np.average(term, weights=sample_weight)
+
+
 class HalfPoissonLoss(BaseLoss):
     """Half Poisson deviance loss with log-link, for regression.
 
@@ -731,14 +792,6 @@ class HalfTweedieLoss(BaseLoss):
     """
 
     def __init__(self, sample_weight=None, power=1.5):
-        check_scalar(
-            power,
-            "power",
-            target_type=numbers.Real,
-            include_boundaries="neither",
-            min_val=-np.inf,
-            max_val=np.inf,
-        )
         super().__init__(
             closs=CyHalfTweedieLoss(power=float(power)),
             link=LogLink(),
@@ -842,6 +895,11 @@ class HalfBinomialLoss(BaseLoss):
     logistic regression, y = [0, 1].
     If you add `constant_to_optimal_zero` to the loss, you get half the
     Bernoulli/binomial deviance.
+
+    More details: Inserting the predicted probability y_pred = expit(raw_prediction)
+    in the loss gives the well known::
+
+        loss(x_i) = - y_true_i * log(y_pred_i) - (1 - y_true_i) * log(1 - y_pred_i)
     """
 
     def __init__(self, sample_weight=None):
@@ -1009,10 +1067,6 @@ class HalfMultinomialLoss(BaseLoss):
         elif proba_out is None:
             proba_out = np.empty_like(gradient_out)
 
-        y_true = ReadonlyArrayWrapper(y_true)
-        raw_prediction = ReadonlyArrayWrapper(raw_prediction)
-        if sample_weight is not None:
-            sample_weight = ReadonlyArrayWrapper(sample_weight)
         return self.closs.gradient_proba(
             y_true=y_true,
             raw_prediction=raw_prediction,
@@ -1023,13 +1077,88 @@ class HalfMultinomialLoss(BaseLoss):
         )
 
 
+class ExponentialLoss(BaseLoss):
+    """Exponential loss with (half) logit link, for binary classification.
+
+    This is also know as boosting loss.
+
+    Domain:
+    y_true in [0, 1], i.e. regression on the unit interval
+    y_pred in (0, 1), i.e. boundaries excluded
+
+    Link:
+    y_pred = expit(2 * raw_prediction)
+
+    For a given sample x_i, the exponential loss is defined as::
+
+        loss(x_i) = y_true_i * exp(-raw_pred_i)) + (1 - y_true_i) * exp(raw_pred_i)
+
+    See:
+    - J. Friedman, T. Hastie, R. Tibshirani.
+      "Additive logistic regression: a statistical view of boosting (With discussion
+      and a rejoinder by the authors)." Ann. Statist. 28 (2) 337 - 407, April 2000.
+      https://doi.org/10.1214/aos/1016218223
+    - A. Buja, W. Stuetzle, Y. Shen. (2005).
+      "Loss Functions for Binary Class Probability Estimation and Classification:
+      Structure and Applications."
+
+    Note that the formulation works for classification, y = {0, 1}, as well as
+    "exponential logistic" regression, y = [0, 1].
+    Note that this is a proper scoring rule, but without it's canonical link.
+
+    More details: Inserting the predicted probability
+    y_pred = expit(2 * raw_prediction) in the loss gives::
+
+        loss(x_i) = y_true_i * sqrt((1 - y_pred_i) / y_pred_i)
+            + (1 - y_true_i) * sqrt(y_pred_i / (1 - y_pred_i))
+    """
+
+    def __init__(self, sample_weight=None):
+        super().__init__(
+            closs=CyExponentialLoss(),
+            link=HalfLogitLink(),
+            n_classes=2,
+        )
+        self.interval_y_true = Interval(0, 1, True, True)
+
+    def constant_to_optimal_zero(self, y_true, sample_weight=None):
+        # This is non-zero only if y_true is neither 0 nor 1.
+        term = -2 * np.sqrt(y_true * (1 - y_true))
+        if sample_weight is not None:
+            term *= sample_weight
+        return term
+
+    def predict_proba(self, raw_prediction):
+        """Predict probabilities.
+
+        Parameters
+        ----------
+        raw_prediction : array of shape (n_samples,) or (n_samples, 1)
+            Raw prediction values (in link space).
+
+        Returns
+        -------
+        proba : array of shape (n_samples, 2)
+            Element-wise class probabilities.
+        """
+        # Be graceful to shape (n_samples, 1) -> (n_samples,)
+        if raw_prediction.ndim == 2 and raw_prediction.shape[1] == 1:
+            raw_prediction = raw_prediction.squeeze(1)
+        proba = np.empty((raw_prediction.shape[0], 2), dtype=raw_prediction.dtype)
+        proba[:, 1] = self.link.inverse(raw_prediction)
+        proba[:, 0] = 1 - proba[:, 1]
+        return proba
+
+
 _LOSSES = {
     "squared_error": HalfSquaredError,
     "absolute_error": AbsoluteError,
     "pinball_loss": PinballLoss,
+    "huber_loss": HuberLoss,
     "poisson_loss": HalfPoissonLoss,
     "gamma_loss": HalfGammaLoss,
     "tweedie_loss": HalfTweedieLoss,
     "binomial_loss": HalfBinomialLoss,
     "multinomial_loss": HalfMultinomialLoss,
+    "exponential_loss": ExponentialLoss,
 }
