@@ -9,14 +9,17 @@ Here are implemented estimators that are resistant to outliers.
 # License: BSD 3 clause
 
 import warnings
-import numbers
+from numbers import Integral, Real
+
 import numpy as np
 from scipy import linalg
 from scipy.stats import chi2
 
-from . import empirical_covariance, EmpiricalCovariance
+from ..base import _fit_context
+from ..utils import check_array, check_random_state
+from ..utils._param_validation import Interval
 from ..utils.extmath import fast_logdet
-from ..utils import check_random_state, check_array
+from ._empirical_covariance import EmpiricalCovariance, empirical_covariance
 
 
 # Minimum Covariance Determinant
@@ -296,7 +299,7 @@ def select_candidates(
     """
     random_state = check_random_state(random_state)
 
-    if isinstance(n_trials, numbers.Integral):
+    if isinstance(n_trials, Integral):
         run_from_estimates = False
     elif isinstance(n_trials, tuple):
         run_from_estimates = True
@@ -605,7 +608,7 @@ class MinCovDet(EmpiricalCovariance):
         MCD estimate. Default is None, which implies that the minimum
         value of support_fraction will be used within the algorithm:
         `(n_sample + n_features + 1) / 2`. The parameter must be in the range
-        (0, 1).
+        (0, 1].
 
     random_state : int, RandomState instance or None, default=None
         Determines the pseudo random number generator for shuffling the data.
@@ -698,6 +701,11 @@ class MinCovDet(EmpiricalCovariance):
     array([0.0813... , 0.0427...])
     """
 
+    _parameter_constraints: dict = {
+        **EmpiricalCovariance._parameter_constraints,
+        "support_fraction": [Interval(Real, 0, 1, closed="right"), None],
+        "random_state": ["random_state"],
+    }
     _nonrobust_covariance = staticmethod(empirical_covariance)
 
     def __init__(
@@ -713,6 +721,7 @@ class MinCovDet(EmpiricalCovariance):
         self.support_fraction = support_fraction
         self.random_state = random_state
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y=None):
         """Fit a Minimum Covariance Determinant with the FastMCD algorithm.
 
