@@ -3,20 +3,23 @@
 # License: BSD 3 clause
 
 import numbers
-import numpy as np
-from scipy.sparse import issparse
+from numbers import Integral, Real
 from warnings import warn
 
+import numpy as np
+from scipy.sparse import issparse
+
+from ..base import OutlierMixin, _fit_context
 from ..tree import ExtraTreeRegressor
+from ..tree._tree import DTYPE as tree_dtype
 from ..utils import (
-    check_random_state,
     check_array,
+    check_random_state,
     gen_batches,
     get_chunk_n_rows,
 )
-from ..utils.validation import check_is_fitted, _num_samples
-from ..base import OutlierMixin
-
+from ..utils._param_validation import Interval, RealNotInt, StrOptions
+from ..utils.validation import _num_samples, check_is_fitted
 from ._bagging import BaseBagging
 
 __all__ = ["IsolationForest"]
@@ -78,7 +81,10 @@ class IsolationForest(OutlierMixin, BaseBagging):
         The number of features to draw from X to train each base estimator.
 
             - If int, then draw `max_features` features.
-            - If float, then draw `max_features * X.shape[1]` features.
+            - If float, then draw `max(1, int(max_features * n_features_in_))` features.
+
+        Note: using a float number less than 1.0 or integer less than number of
+        features will enable feature subsampling and leads to a longer runtime.
 
     bootstrap : bool, default=False
         If True, individual trees are fit on random subsets of the training
@@ -110,10 +116,21 @@ class IsolationForest(OutlierMixin, BaseBagging):
 
     Attributes
     ----------
+    estimator_ : :class:`~sklearn.tree.ExtraTreeRegressor` instance
+        The child estimator template used to create the collection of
+        fitted sub-estimators.
+
+        .. versionadded:: 1.2
+           `base_estimator_` was renamed to `estimator_`.
+
     base_estimator_ : ExtraTreeRegressor instance
         The child estimator template used to create the collection of
         fitted sub-estimators.
 
+        .. deprecated:: 1.2
+            `base_estimator_` is deprecated and will be removed in 1.4.
+            Use `estimator_` instead.
+
     estimators_ : list of ExtraTreeRegressor instances
         The collection of fitted sub-estimators.
 
@@ -139,13 +156,6 @@ class IsolationForest(OutlierMixin, BaseBagging):
 
         .. versionadded:: 0.20
 
-    n_features_ : int
-        The number of features when ``fit`` is performed.
-
-        .. deprecated:: 1.0
-            Attribute `n_features_` was deprecated in version 1.0 and will be
-            removed in 1.2. Use `n_features_in_` instead.
-
     n_features_in_ : int
         Number of features seen during :term:`fit`.
 
@@ -191,6 +201,28 @@ class IsolationForest(OutlierMixin, BaseBagging):
     array([ 1,  1, -1])
     """
 
+    _parameter_constraints: dict = {
+        "n_estimators": [Interval(Integral, 1, None, closed="left")],
+        "max_samples": [
+            StrOptions({"auto"}),
+            Interval(Integral, 1, None, closed="left"),
+            Interval(RealNotInt, 0, 1, closed="right"),
+        ],
+        "contamination": [
+            StrOptions({"auto"}),
+            Interval(Real, 0, 0.5, closed="right"),
+        ],
+        "max_features": [
+            Integral,
+            Interval(Real, 0, 1, closed="right"),
+        ],
+        "bootstrap": ["boolean"],
+        "n_jobs": [Integral, None],
+        "random_state": ["random_state"],
+        "verbose": ["verbose"],
+        "warm_start": ["boolean"],
+    }
+
     def __init__(
         self,
         *,
@@ -205,7 +237,7 @@ class IsolationForest(OutlierMixin, BaseBagging):
         warm_start=False,
     ):
         super().__init__(
-            base_estimator=ExtraTreeRegressor(
+            estimator=ExtraTreeRegressor(
                 max_features=1, splitter="random", random_state=random_state
             ),
             # here above max_features has no links with self.max_features
@@ -232,6 +264,7 @@ class IsolationForest(OutlierMixin, BaseBagging):
         # copies.
         return {"prefer": "threads"}
 
+    @_fit_context(prefer_skip_nested_validation=True)
     def fit(self, X, y=None, sample_weight=None):
         """
         Fit estimator.
@@ -254,7 +287,7 @@ class IsolationForest(OutlierMixin, BaseBagging):
         self : object
             Fitted estimator.
         """
-        X = self._validate_data(X, accept_sparse=["csc"])
+        X = self._validate_data(X, accept_sparse=["csc"], dtype=tree_dtype)
         if issparse(X):
             # Pre-sort indices to avoid that each individual tree of the
             # ensemble sorts the indices.
@@ -266,22 +299,8 @@ class IsolationForest(OutlierMixin, BaseBagging):
         # ensure that max_sample is in [1, n_samples]:
         n_samples = X.shape[0]
 
-        if self.contamination != "auto":
-            if not (0.0 < self.contamination <= 0.5):
-                raise ValueError(
-                    "contamination must be in (0, 0.5], got: %f" % self.contamination
-                )
-
-        if isinstance(self.max_samples, str):
-            if self.max_samples == "auto":
-                max_samples = min(256, n_samples)
-            else:
-                raise ValueError(
-                    "max_samples (%s) is not supported."
-                    'Valid choices are: "auto", int or'
-                    "float"
-                    % self.max_samples
-                )
+        if isinstance(self.max_samples, str) and self.max_samples == "auto":
+            max_samples = min(256, n_samples)
 
         elif isinstance(self.max_samples, numbers.Integral):
             if self.max_samples > n_samples:
@@ -294,11 +313,7 @@ class IsolationForest(OutlierMixin, BaseBagging):
                 max_samples = n_samples
             else:
                 max_samples = self.max_samples
-        else:  # float
-            if not 0.0 < self.max_samples <= 1.0:
-                raise ValueError(
-                    "max_samples must be in (0, 1], got %r" % self.max_samples
-                )
+        else:  # max_samples is float
             max_samples = int(self.max_samples * X.shape[0])
 
         self.max_samples_ = max_samples
@@ -312,14 +327,26 @@ class IsolationForest(OutlierMixin, BaseBagging):
             check_input=False,
         )
 
+        self._average_path_length_per_tree, self._decision_path_lengths = zip(
+            *[
+                (
+                    _average_path_length(tree.tree_.n_node_samples),
+                    tree.tree_.compute_node_depths(),
+                )
+                for tree in self.estimators_
+            ]
+        )
+
         if self.contamination == "auto":
             # 0.5 plays a special role as described in the original paper.
             # we take the opposite as we consider the opposite of their score.
             self.offset_ = -0.5
             return self
 
-        # else, define offset_ wrt contamination parameter
-        self.offset_ = np.percentile(self.score_samples(X), 100.0 * self.contamination)
+        # Else, define offset_ wrt contamination parameter
+        # To avoid performing input validation a second time we call
+        # _score_samples rather than score_samples
+        self.offset_ = np.percentile(self._score_samples(X), 100.0 * self.contamination)
 
         return self
 
@@ -402,19 +429,24 @@ class IsolationForest(OutlierMixin, BaseBagging):
             The anomaly score of the input samples.
             The lower, the more abnormal.
         """
-        # code structure from ForestClassifier/predict_proba
+        # Check data
+        X = self._validate_data(X, accept_sparse="csr", dtype=np.float32, reset=False)
+
+        return self._score_samples(X)
+
+    def _score_samples(self, X):
+        """Private version of score_samples without input validation.
+
+        Input validation would remove feature names, so we disable it.
+        """
+        # Code structure from ForestClassifier/predict_proba
 
         check_is_fitted(self)
 
-        # Check data
-        X = self._validate_data(X, accept_sparse="csr", reset=False)
-
-        # Take the opposite of the scores as bigger is better (here less
-        # abnormal)
+        # Take the opposite of the scores as bigger is better (here less abnormal)
         return -self._compute_chunked_score_samples(X)
 
     def _compute_chunked_score_samples(self, X):
-
         n_samples = _num_samples(X)
 
         if self._max_features == X.shape[1]:
@@ -462,19 +494,21 @@ class IsolationForest(OutlierMixin, BaseBagging):
 
         depths = np.zeros(n_samples, order="f")
 
-        for tree, features in zip(self.estimators_, self.estimators_features_):
+        average_path_length_max_samples = _average_path_length([self._max_samples])
+
+        for tree_idx, (tree, features) in enumerate(
+            zip(self.estimators_, self.estimators_features_)
+        ):
             X_subset = X[:, features] if subsample_features else X
 
-            leaves_index = tree.apply(X_subset)
-            node_indicator = tree.decision_path(X_subset)
-            n_samples_leaf = tree.tree_.n_node_samples[leaves_index]
+            leaves_index = tree.apply(X_subset, check_input=False)
 
             depths += (
-                np.ravel(node_indicator.sum(axis=1))
-                + _average_path_length(n_samples_leaf)
+                self._decision_path_lengths[tree_idx][leaves_index]
+                + self._average_path_length_per_tree[tree_idx][leaves_index]
                 - 1.0
             )
-        denominator = len(self.estimators_) * _average_path_length([self.max_samples_])
+        denominator = len(self.estimators_) * average_path_length_max_samples
         scores = 2 ** (
             # For a single training sample, denominator and depth are 0.
             # Therefore, we set the score manually to 1.
