@@ -1,17 +1,19 @@
+from abc import abstractmethod
 from copy import deepcopy
 from math import ceil, floor, log
-from abc import abstractmethod
-from numbers import Integral
+from numbers import Integral, Real
 
 import numpy as np
-from ._search import BaseSearchCV
-from . import ParameterGrid, ParameterSampler
-from ..base import is_classifier
-from ._split import check_cv, _yields_constant_splits
+
+from ..base import _fit_context, is_classifier
+from ..metrics._scorer import get_scorer_names
 from ..utils import resample
+from ..utils._param_validation import Interval, StrOptions
 from ..utils.multiclass import check_classification_targets
 from ..utils.validation import _num_samples
-
+from . import ParameterGrid, ParameterSampler
+from ._search import BaseSearchCV
+from ._split import _yields_constant_splits, check_cv
 
 __all__ = ["HalvingGridSearchCV", "HalvingRandomSearchCV"]
 
@@ -50,7 +52,11 @@ def _top_k(results, k, itr):
         for a in (results["iter"], results["mean_test_score"], results["params"])
     )
     iter_indices = np.flatnonzero(iteration == itr)
-    sorted_indices = np.argsort(mean_test_score[iter_indices])
+    scores = mean_test_score[iter_indices]
+    # argsort() places NaNs at the end of the array so we move NaNs to the
+    # front of the array so the last `k` items are the those with the
+    # highest scores.
+    sorted_indices = np.roll(np.argsort(scores), np.count_nonzero(np.isnan(scores)))
     return np.array(params[iter_indices][sorted_indices[-k:]])
 
 
@@ -62,6 +68,25 @@ class BaseSuccessiveHalving(BaseSearchCV):
     Zohar Karnin, Tomer Koren, Oren Somekh
     """
 
+    _parameter_constraints: dict = {
+        **BaseSearchCV._parameter_constraints,
+        # overwrite `scoring` since multi-metrics are not supported
+        "scoring": [StrOptions(set(get_scorer_names())), callable, None],
+        "random_state": ["random_state"],
+        "max_resources": [
+            Interval(Integral, 0, None, closed="neither"),
+            StrOptions({"auto"}),
+        ],
+        "min_resources": [
+            Interval(Integral, 0, None, closed="neither"),
+            StrOptions({"exhaust", "smallest"}),
+        ],
+        "resource": [str],
+        "factor": [Interval(Real, 0, None, closed="neither")],
+        "aggressive_elimination": ["boolean"],
+    }
+    _parameter_constraints.pop("pre_dispatch")  # not used in this class
+
     def __init__(
         self,
         estimator,
@@ -99,16 +124,6 @@ class BaseSuccessiveHalving(BaseSearchCV):
         self.aggressive_elimination = aggressive_elimination
 
     def _check_input_parameters(self, X, y, groups):
-
-        if self.scoring is not None and not (
-            isinstance(self.scoring, str) or callable(self.scoring)
-        ):
-            raise ValueError(
-                "scoring parameter must be a string, "
-                "a callable or None. Multimetric scoring is not "
-                "supported."
-            )
-
         # We need to enforce that successive calls to cv.split() yield the same
         # splits: see https://github.com/scikit-learn/scikit-learn/issues/15149
         if not _yields_constant_splits(self._checked_cv_orig):
@@ -127,26 +142,6 @@ class BaseSuccessiveHalving(BaseSearchCV):
                 f"by estimator {self.estimator.__class__.__name__}"
             )
 
-        if isinstance(self.max_resources, str) and self.max_resources != "auto":
-            raise ValueError(
-                "max_resources must be either 'auto' or a positive integer"
-            )
-        if self.max_resources != "auto" and (
-            not isinstance(self.max_resources, Integral) or self.max_resources <= 0
-        ):
-            raise ValueError(
-                "max_resources must be either 'auto' or a positive integer"
-            )
-
-        if self.min_resources not in ("smallest", "exhaust") and (
-            not isinstance(self.min_resources, Integral) or self.min_resources <= 0
-        ):
-            raise ValueError(
-                "min_resources must be either 'smallest', 'exhaust', "
-                "or a positive integer "
-                "no greater than max_resources."
-            )
-
         if isinstance(self, HalvingRandomSearchCV):
             if self.min_resources == self.n_candidates == "exhaust":
                 # for n_candidates=exhaust to work, we need to know what
@@ -155,12 +150,6 @@ class BaseSuccessiveHalving(BaseSearchCV):
                 raise ValueError(
                     "n_candidates and min_resources cannot be both set to 'exhaust'."
                 )
-            if self.n_candidates != "exhaust" and (
-                not isinstance(self.n_candidates, Integral) or self.n_candidates <= 0
-            ):
-                raise ValueError(
-                    "n_candidates must be either 'exhaust' or a positive integer"
-                )
 
         self.min_resources_ = self.min_resources
         if self.min_resources_ in ("smallest", "exhaust"):
@@ -183,7 +172,7 @@ class BaseSuccessiveHalving(BaseSearchCV):
         if self.max_resources_ == "auto":
             if not self.resource == "n_samples":
                 raise ValueError(
-                    "max_resources can only be 'auto' if resource='n_samples'"
+                    "resource can only be 'n_samples' when max_resources='auto'"
                 )
             self.max_resources_ = _num_samples(X)
 
@@ -199,11 +188,6 @@ class BaseSuccessiveHalving(BaseSearchCV):
                 "an empty dataset X."
             )
 
-        if not isinstance(self.refit, bool):
-            raise ValueError(
-                f"refit is expected to be a boolean. Got {type(self.refit)} instead."
-            )
-
     @staticmethod
     def _select_best_index(refit, refit_metric, results):
         """Custom refit callable to return the index of the best candidate.
@@ -216,9 +200,21 @@ class BaseSuccessiveHalving(BaseSearchCV):
         """
         last_iter = np.max(results["iter"])
         last_iter_indices = np.flatnonzero(results["iter"] == last_iter)
-        best_idx = np.argmax(results["mean_test_score"][last_iter_indices])
+
+        test_scores = results["mean_test_score"][last_iter_indices]
+        # If all scores are NaNs there is no way to pick between them,
+        # so we (arbitrarily) declare the zero'th entry the best one
+        if np.isnan(test_scores).all():
+            best_idx = 0
+        else:
+            best_idx = np.nanargmax(test_scores)
+
         return last_iter_indices[best_idx]
 
+    @_fit_context(
+        # Halving*SearchCV.estimator is not validated yet
+        prefer_skip_nested_validation=False
+    )
     def fit(self, X, y=None, groups=None, **fit_params):
         """Run fit with all sets of parameters.
 
@@ -318,7 +314,6 @@ class BaseSuccessiveHalving(BaseSearchCV):
         self.n_candidates_ = []
 
         for itr in range(n_iterations):
-
             power = itr  # default
             if self.aggressive_elimination:
                 # this will set n_resources to the initial value (i.e. the
@@ -655,6 +650,8 @@ class HalvingGridSearchCV(BaseSuccessiveHalving):
     The parameters selected are those that maximize the score of the held-out
     data, according to the scoring parameter.
 
+    All parameter combinations scored with a NaN will share the lowest rank.
+
     Examples
     --------
 
@@ -677,6 +674,11 @@ class HalvingGridSearchCV(BaseSuccessiveHalving):
 
     _required_parameters = ["estimator", "param_grid"]
 
+    _parameter_constraints: dict = {
+        **BaseSuccessiveHalving._parameter_constraints,
+        "param_grid": [dict, list],
+    }
+
     def __init__(
         self,
         estimator,
@@ -748,13 +750,15 @@ class HalvingRandomSearchCV(BaseSuccessiveHalving):
         Either estimator needs to provide a ``score`` function,
         or ``scoring`` must be passed.
 
-    param_distributions : dict
-        Dictionary with parameters names (string) as keys and distributions
+    param_distributions : dict or list of dicts
+        Dictionary with parameters names (`str`) as keys and distributions
         or lists of parameters to try. Distributions must provide a ``rvs``
         method for sampling (such as those from scipy.stats.distributions).
         If a list is given, it is sampled uniformly.
+        If a list of dicts is given, first a dict is sampled uniformly, and
+        then a parameter is sampled using that dict as above.
 
-    n_candidates : int, default='exhaust'
+    n_candidates : "exhaust" or int, default="exhaust"
         The number of candidate parameters to sample, at the first
         iteration. Using 'exhaust' will sample enough candidates so that the
         last iteration uses as many resources as possible, based on
@@ -992,6 +996,8 @@ class HalvingRandomSearchCV(BaseSuccessiveHalving):
     The parameters selected are those that maximize the score of the held-out
     data, according to the scoring parameter.
 
+    All parameter combinations scored with a NaN will share the lowest rank.
+
     Examples
     --------
 
@@ -1018,6 +1024,15 @@ class HalvingRandomSearchCV(BaseSuccessiveHalving):
 
     _required_parameters = ["estimator", "param_distributions"]
 
+    _parameter_constraints: dict = {
+        **BaseSuccessiveHalving._parameter_constraints,
+        "param_distributions": [dict, list],
+        "n_candidates": [
+            Interval(Integral, 0, None, closed="neither"),
+            StrOptions({"exhaust"}),
+        ],
+    }
+
     def __init__(
         self,
         estimator,
